"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.__testing__ = exports.lookupReplaceCost = exports.prettyPrintWeightMap = exports.prettyPrintSwap = exports.prettyPrintReplace = exports.addDefToWeightMap = exports.createWeightMap = void 0;
function createWeightMap(...defs) {
    const map = _createWeightMap();
    _addDefToWeightMap(map, ...defs);
    return map;
}
exports.createWeightMap = createWeightMap;
function addDefToWeightMap(map, ...defs) {
    return _addDefToWeightMap(map, ...defs);
}
exports.addDefToWeightMap = addDefToWeightMap;
function _addDefToWeightMap(map, ...defs) {
    function addSet(set, def) {
        addSetToTrieCost(map.insDel, set, def.insDel);
        addSetToTrieTrieCost(map.replace, set, def.replace);
        addSetToTrieTrieCost(map.swap, set, def.swap);
    }
    for (const def of defs) {
        const mapSets = splitMap(def);
        mapSets.forEach((s) => addSet(s, def));
    }
    return map;
}
function _createWeightMap() {
    return new _WeightedMap();
}
function lowest(a, b) {
    if (a === undefined)
        return b;
    if (b === undefined)
        return a;
    return a <= b ? a : b;
}
/**
 * Splits a WeightedMapDef.map
 * @param map
 */
function splitMap(def) {
    const { map } = def;
    const sets = map.split('|');
    return sets.map(splitMapSubstrings).filter((s) => s.length > 0);
}
function splitMapSubstrings(map) {
    const values = [];
    const len = map.length;
    for (let i = 0; i < len; ++i) {
        const c = map[i];
        if (c !== '(') {
            values.push(c);
            continue;
        }
        const s = i + 1;
        while (map[++i] !== ')' && i < len) {
            // empty
        }
        values.push(map.slice(s, i));
    }
    return values.map((s) => s.trim()).filter((s) => !!s);
}
function addToTrieCost(trie, str, cost) {
    if (!str)
        return;
    let t = trie;
    for (const c of str) {
        const n = (t.n = t.n || Object.create(null));
        t = n[c] = n[c] || Object.create(null);
    }
    t.c = lowest(t.c, cost);
}
function addToTrieTrieCost(trie, left, right, cost) {
    let t = trie;
    for (const c of left) {
        const n = (t.n = t.n || Object.create(null));
        t = n[c] = n[c] || Object.create(null);
    }
    const trieCost = (t.t = t.t || Object.create(null));
    addToTrieCost(trieCost, right, cost);
}
function addSetToTrieCost(trie, set, cost) {
    if (cost === undefined)
        return;
    for (const str of set) {
        addToTrieCost(trie, str, cost);
    }
}
function addSetToTrieTrieCost(trie, set, cost) {
    if (cost === undefined)
        return;
    for (const left of set) {
        for (const right of set) {
            if (left === right)
                continue;
            addToTrieTrieCost(trie, left, right, cost);
        }
    }
}
function* searchTrieNodes(trie, str, i) {
    const len = str.length;
    for (let n = trie.n; i < len && n;) {
        const t = n[str[i]];
        if (!t)
            return;
        ++i;
        yield { i, t };
        n = t.n;
    }
}
function* walkTrieNodes(t, s) {
    if (!t)
        return;
    yield { s, t };
    if (!t.n)
        return;
    for (const [k, v] of Object.entries(t.n)) {
        yield* walkTrieNodes(v, s + k);
    }
}
function* walkTrieCost(trie) {
    for (const { s, t } of walkTrieNodes(trie, '')) {
        if (t.c) {
            yield { s, c: t.c };
        }
    }
}
function* walkTrieTrieCost(trie) {
    for (const { s: a, t } of walkTrieNodes(trie, '')) {
        if (t.t) {
            for (const { s: b, c } of walkTrieCost(t.t)) {
                yield { a, b, c };
            }
        }
    }
}
function* findTrieCostPrefixes(trie, str, i) {
    for (const n of searchTrieNodes(trie, str, i)) {
        const c = n.t.c;
        if (c !== undefined) {
            yield { i: n.i, c };
        }
    }
}
function* findTrieTrieCostPrefixes(trie, str, i) {
    for (const n of searchTrieNodes(trie, str, i)) {
        const t = n.t.t;
        if (t !== undefined) {
            yield { i: n.i, t };
        }
    }
}
class _WeightedMap {
    constructor() {
        this.insDel = {};
        this.replace = {};
        this.swap = {};
    }
    *calcInsDelCosts(pos) {
        const { a, ai, b, bi, c } = pos;
        for (const del of findTrieCostPrefixes(this.insDel, a, ai)) {
            yield { a, b, ai: del.i, bi, c: c + del.c };
        }
        for (const ins of findTrieCostPrefixes(this.insDel, b, bi)) {
            yield { a, b, ai, bi: ins.i, c: c + ins.c };
        }
    }
    *calcReplaceCosts(pos) {
        // Search for matching substrings in `a` to be replaced by
        // matching substrings from `b`. All substrings start at their
        // respective `ai`/`bi` positions.
        const { a, ai, b, bi, c } = pos;
        for (const del of findTrieTrieCostPrefixes(this.replace, a, ai)) {
            for (const ins of findTrieCostPrefixes(del.t, b, bi)) {
                yield { a, b, ai: del.i, bi: ins.i, c: c + ins.c };
            }
        }
    }
    *calcSwapCosts(pos) {
        const { a, ai, b, bi, c } = pos;
        const swap = this.swap;
        for (const left of findTrieTrieCostPrefixes(swap, a, ai)) {
            for (const right of findTrieCostPrefixes(left.t, a, left.i)) {
                const sw = a.slice(left.i, right.i) + a.slice(ai, left.i);
                if (b.slice(bi).startsWith(sw)) {
                    const len = sw.length;
                    yield { a, b, ai: ai + len, bi: bi + len, c: c + right.c };
                }
            }
        }
    }
}
function prettyPrintInsDel(trie, pfx = '', indent = '  ') {
    function* walk() {
        for (const { s, c } of walkTrieCost(trie)) {
            yield indent + `(${s}) = ${c}`;
        }
    }
    return ['InsDel:', ...[...walk()].sort()].map((line) => pfx + line + '\n').join('');
}
function prettyPrintReplace(trie, pfx = '', indent = '  ') {
    function* walk() {
        for (const { a, b, c } of walkTrieTrieCost(trie)) {
            yield indent + `(${a}) -> (${b}) = ${c}`;
        }
    }
    return ['Replace:', ...[...walk()].sort()].map((line) => pfx + line + '\n').join('');
}
exports.prettyPrintReplace = prettyPrintReplace;
function prettyPrintSwap(trie, pfx = '', indent = '  ') {
    function* walk() {
        for (const { a, b, c } of walkTrieTrieCost(trie)) {
            yield indent + `(${a}) <-> (${b}) = ${c}`;
        }
    }
    return ['Swap:', ...[...walk()].sort()].map((line) => pfx + line + '\n').join('');
}
exports.prettyPrintSwap = prettyPrintSwap;
function prettyPrintWeightMap(map) {
    return [prettyPrintInsDel(map.insDel), prettyPrintReplace(map.replace), prettyPrintSwap(map.swap)].join('\n');
}
exports.prettyPrintWeightMap = prettyPrintWeightMap;
function lookupReplaceCost(map, a, b) {
    var _a, _b;
    const trie = map.replace;
    let tt = trie;
    for (let ai = 0; ai < a.length && tt; ++ai) {
        tt = (_a = tt.n) === null || _a === void 0 ? void 0 : _a[a[ai]];
    }
    if (!tt)
        return undefined;
    let t = tt.t;
    for (let bi = 0; bi < b.length && t; ++bi) {
        t = (_b = t.n) === null || _b === void 0 ? void 0 : _b[b[bi]];
    }
    return t === null || t === void 0 ? void 0 : t.c;
}
exports.lookupReplaceCost = lookupReplaceCost;
exports.__testing__ = {
    splitMap,
    splitMapSubstrings,
    findTrieCostPrefixes,
    findTrieTrieCostPrefixes,
};
//# sourceMappingURL=weightedMaps.js.map